package com.serbia.webadmin.app.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.serbia.common.log.annotation.OperationLog;
import com.serbia.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.serbia.webadmin.app.vo.*;
import com.serbia.webadmin.app.dto.*;
import com.serbia.webadmin.app.model.*;
import com.serbia.webadmin.app.service.*;
import com.serbia.common.core.object.*;
import com.serbia.common.core.util.*;
import com.serbia.common.core.constant.*;
import com.serbia.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 国家区域编码管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-12-10
 */
@Tag(name = "国家区域编码管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/serbiaCountryCode")
public class SerbiaCountryCodeController {

    @Autowired
    private SerbiaCountryCodeService serbiaCountryCodeService;

    /**
     * 新增国家区域编码管理数据。
     *
     * @param serbiaCountryCodeDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"serbiaCountryCodeDto.id"})
    @SaCheckPermission("serbiaCountryCode.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody SerbiaCountryCodeDto serbiaCountryCodeDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(serbiaCountryCodeDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        SerbiaCountryCode serbiaCountryCode = MyModelUtil.copyTo(serbiaCountryCodeDto, SerbiaCountryCode.class);
        serbiaCountryCode = serbiaCountryCodeService.saveNew(serbiaCountryCode);
        return ResponseResult.success(serbiaCountryCode.getId());
    }

    /**
     * 更新国家区域编码管理数据。
     *
     * @param serbiaCountryCodeDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("serbiaCountryCode.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody SerbiaCountryCodeDto serbiaCountryCodeDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(serbiaCountryCodeDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        SerbiaCountryCode serbiaCountryCode = MyModelUtil.copyTo(serbiaCountryCodeDto, SerbiaCountryCode.class);
        SerbiaCountryCode originalSerbiaCountryCode = serbiaCountryCodeService.getById(serbiaCountryCode.getId());
        if (originalSerbiaCountryCode == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!serbiaCountryCodeService.update(serbiaCountryCode, originalSerbiaCountryCode)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除国家区域编码管理数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("serbiaCountryCode.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除国家区域编码管理数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("serbiaCountryCode.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的国家区域编码管理列表。
     *
     * @param serbiaCountryCodeDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("serbiaCountryCode.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<SerbiaCountryCodeVo>> list(
            @MyRequestBody SerbiaCountryCodeDto serbiaCountryCodeDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        SerbiaCountryCode serbiaCountryCodeFilter = MyModelUtil.copyTo(serbiaCountryCodeDtoFilter, SerbiaCountryCode.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, SerbiaCountryCode.class);
        List<SerbiaCountryCode> serbiaCountryCodeList =
                serbiaCountryCodeService.getSerbiaCountryCodeListWithRelation(serbiaCountryCodeFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(serbiaCountryCodeList, SerbiaCountryCodeVo.class));
    }

    /**
     * 查看指定国家区域编码管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("serbiaCountryCode.view")
    @GetMapping("/view")
    public ResponseResult<SerbiaCountryCodeVo> view(@RequestParam Long id) {
        SerbiaCountryCode serbiaCountryCode = serbiaCountryCodeService.getByIdWithRelation(id, MyRelationParam.full());
        if (serbiaCountryCode == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        SerbiaCountryCodeVo serbiaCountryCodeVo = MyModelUtil.copyTo(serbiaCountryCode, SerbiaCountryCodeVo.class);
        return ResponseResult.success(serbiaCountryCodeVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        SerbiaCountryCode originalSerbiaCountryCode = serbiaCountryCodeService.getById(id);
        if (originalSerbiaCountryCode == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!serbiaCountryCodeService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
