package com.serbia.webadmin.app.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.serbia.common.log.annotation.OperationLog;
import com.serbia.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.serbia.webadmin.app.vo.*;
import com.serbia.webadmin.app.dto.*;
import com.serbia.webadmin.app.model.*;
import com.serbia.webadmin.app.service.*;
import com.serbia.common.core.object.*;
import com.serbia.common.core.util.*;
import com.serbia.common.core.constant.*;
import com.serbia.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 聊天群组成员管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-12-10
 */
@Tag(name = "聊天群组成员管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/serbiaGroupMembers")
public class SerbiaGroupMembersController {

    @Autowired
    private SerbiaGroupMembersService serbiaGroupMembersService;

    /**
     * 新增聊天群组成员管理数据。
     *
     * @param serbiaGroupMembersDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"serbiaGroupMembersDto.id"})
    @SaCheckPermission("serbiaGroupMembers.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody SerbiaGroupMembersDto serbiaGroupMembersDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(serbiaGroupMembersDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        SerbiaGroupMembers serbiaGroupMembers = MyModelUtil.copyTo(serbiaGroupMembersDto, SerbiaGroupMembers.class);
        // 验证关联Id的数据合法性
        CallResult callResult = serbiaGroupMembersService.verifyRelatedData(serbiaGroupMembers, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        serbiaGroupMembers = serbiaGroupMembersService.saveNew(serbiaGroupMembers);
        return ResponseResult.success(serbiaGroupMembers.getId());
    }

    /**
     * 更新聊天群组成员管理数据。
     *
     * @param serbiaGroupMembersDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("serbiaGroupMembers.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody SerbiaGroupMembersDto serbiaGroupMembersDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(serbiaGroupMembersDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        SerbiaGroupMembers serbiaGroupMembers = MyModelUtil.copyTo(serbiaGroupMembersDto, SerbiaGroupMembers.class);
        SerbiaGroupMembers originalSerbiaGroupMembers = serbiaGroupMembersService.getById(serbiaGroupMembers.getId());
        if (originalSerbiaGroupMembers == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = serbiaGroupMembersService.verifyRelatedData(serbiaGroupMembers, originalSerbiaGroupMembers);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!serbiaGroupMembersService.update(serbiaGroupMembers, originalSerbiaGroupMembers)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除聊天群组成员管理数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("serbiaGroupMembers.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除聊天群组成员管理数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("serbiaGroupMembers.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的聊天群组成员管理列表。
     *
     * @param serbiaGroupMembersDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("serbiaGroupMembers.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<SerbiaGroupMembersVo>> list(
            @MyRequestBody SerbiaGroupMembersDto serbiaGroupMembersDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        SerbiaGroupMembers serbiaGroupMembersFilter = MyModelUtil.copyTo(serbiaGroupMembersDtoFilter, SerbiaGroupMembers.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, SerbiaGroupMembers.class);
        List<SerbiaGroupMembers> serbiaGroupMembersList =
                serbiaGroupMembersService.getSerbiaGroupMembersListWithRelation(serbiaGroupMembersFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(serbiaGroupMembersList, SerbiaGroupMembersVo.class));
    }

    /**
     * 查看指定聊天群组成员管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("serbiaGroupMembers.view")
    @GetMapping("/view")
    public ResponseResult<SerbiaGroupMembersVo> view(@RequestParam Long id) {
        SerbiaGroupMembers serbiaGroupMembers = serbiaGroupMembersService.getByIdWithRelation(id, MyRelationParam.full());
        if (serbiaGroupMembers == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        SerbiaGroupMembersVo serbiaGroupMembersVo = MyModelUtil.copyTo(serbiaGroupMembers, SerbiaGroupMembersVo.class);
        return ResponseResult.success(serbiaGroupMembersVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        SerbiaGroupMembers originalSerbiaGroupMembers = serbiaGroupMembersService.getById(id);
        if (originalSerbiaGroupMembers == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!serbiaGroupMembersService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
