package com.serbia.webadmin.app.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.ReflectUtil;
import com.serbia.common.core.upload.BaseUpDownloader;
import com.serbia.common.core.upload.UpDownloaderFactory;
import com.serbia.common.core.upload.UploadResponseInfo;
import com.serbia.common.core.upload.UploadStoreInfo;
import com.serbia.common.log.annotation.OperationLog;
import com.serbia.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.serbia.webadmin.app.vo.*;
import com.serbia.webadmin.app.dto.*;
import com.serbia.webadmin.app.model.*;
import com.serbia.webadmin.app.service.*;
import com.serbia.common.core.object.*;
import com.serbia.common.core.util.*;
import com.serbia.common.core.constant.*;
import com.serbia.common.core.annotation.MyRequestBody;
import com.serbia.common.redis.cache.SessionCacheHelper;
import com.serbia.webadmin.config.ApplicationConfig;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;

/**
 * 聊天群组管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-12-10
 */
@Tag(name = "聊天群组管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/serbiaGroups")
public class SerbiaGroupsController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private SessionCacheHelper cacheHelper;
    @Autowired
    private UpDownloaderFactory upDownloaderFactory;
    @Autowired
    private SerbiaGroupsService serbiaGroupsService;

    /**
     * 新增聊天群组管理数据。
     *
     * @param serbiaGroupsDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {
            "serbiaGroupsDto.groupId",
            "serbiaGroupsDto.searchString",
            "serbiaGroupsDto.createTimeStart",
            "serbiaGroupsDto.createTimeEnd",
            "serbiaGroupsDto.updateTimeStart",
            "serbiaGroupsDto.updateTimeEnd"})
    @SaCheckPermission("serbiaGroups.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody SerbiaGroupsDto serbiaGroupsDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(serbiaGroupsDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        SerbiaGroups serbiaGroups = MyModelUtil.copyTo(serbiaGroupsDto, SerbiaGroups.class);
        // 验证关联Id的数据合法性
        CallResult callResult = serbiaGroupsService.verifyRelatedData(serbiaGroups, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        serbiaGroups = serbiaGroupsService.saveNew(serbiaGroups);
        return ResponseResult.success(serbiaGroups.getGroupId());
    }

    /**
     * 更新聊天群组管理数据。
     *
     * @param serbiaGroupsDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {
            "serbiaGroupsDto.searchString",
            "serbiaGroupsDto.createTimeStart",
            "serbiaGroupsDto.createTimeEnd",
            "serbiaGroupsDto.updateTimeStart",
            "serbiaGroupsDto.updateTimeEnd"})
    @SaCheckPermission("serbiaGroups.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody SerbiaGroupsDto serbiaGroupsDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(serbiaGroupsDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        SerbiaGroups serbiaGroups = MyModelUtil.copyTo(serbiaGroupsDto, SerbiaGroups.class);
        SerbiaGroups originalSerbiaGroups = serbiaGroupsService.getById(serbiaGroups.getGroupId());
        if (originalSerbiaGroups == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = serbiaGroupsService.verifyRelatedData(serbiaGroups, originalSerbiaGroups);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!serbiaGroupsService.update(serbiaGroups, originalSerbiaGroups)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除聊天群组管理数据。
     *
     * @param groupId 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("serbiaGroups.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long groupId) {
        if (MyCommonUtil.existBlankArgument(groupId)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(groupId);
    }

    /**
     * 批量删除聊天群组管理数据。
     *
     * @param groupIdList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("serbiaGroups.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> groupIdList) {
        if (MyCommonUtil.existBlankArgument(groupIdList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long groupId : groupIdList) {
            ResponseResult<Void> responseResult = this.doDelete(groupId);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的聊天群组管理列表。
     *
     * @param serbiaGroupsDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("serbiaGroups.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<SerbiaGroupsVo>> list(
            @MyRequestBody SerbiaGroupsDto serbiaGroupsDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        SerbiaGroups serbiaGroupsFilter = MyModelUtil.copyTo(serbiaGroupsDtoFilter, SerbiaGroups.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, SerbiaGroups.class);
        List<SerbiaGroups> serbiaGroupsList =
                serbiaGroupsService.getSerbiaGroupsListWithRelation(serbiaGroupsFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(serbiaGroupsList, SerbiaGroupsVo.class));
    }

    /**
     * 查看指定聊天群组管理对象详情。
     *
     * @param groupId 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("serbiaGroups.view")
    @GetMapping("/view")
    public ResponseResult<SerbiaGroupsVo> view(@RequestParam Long groupId) {
        SerbiaGroups serbiaGroups = serbiaGroupsService.getByIdWithRelation(groupId, MyRelationParam.full());
        if (serbiaGroups == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        SerbiaGroupsVo serbiaGroupsVo = MyModelUtil.copyTo(serbiaGroups, SerbiaGroupsVo.class);
        return ResponseResult.success(serbiaGroupsVo);
    }

    /**
     * 附件文件下载。
     * 这里将图片和其他类型的附件文件放到不同的父目录下，主要为了便于今后图片文件的迁移。
     *
     * @param groupId 附件所在记录的主键Id。
     * @param fieldName 附件所属的字段名。
     * @param filename  文件名。如果没有提供该参数，就从当前记录的指定字段中读取。
     * @param asImage   下载文件是否为图片。
     * @param response  Http 应答对象。
     */
    @SaCheckPermission("serbiaGroups.view")
    @OperationLog(type = SysOperationLogType.DOWNLOAD, saveResponse = false)
    @GetMapping("/download")
    public void download(
            @RequestParam(required = false) Long groupId,
            @RequestParam String fieldName,
            @RequestParam String filename,
            @RequestParam Boolean asImage,
            HttpServletResponse response) {
        if (MyCommonUtil.existBlankArgument(fieldName, filename, asImage)) {
            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            return;
        }
        // 使用try来捕获异常，是为了保证一旦出现异常可以返回500的错误状态，便于调试。
        // 否则有可能给前端返回的是200的错误码。
        try {
            // 如果请求参数中没有包含主键Id，就判断该文件是否为当前session上传的。
            if (groupId == null) {
                if (!cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_FORBIDDEN);
                    return;
                }
            } else {
                SerbiaGroups serbiaGroups = serbiaGroupsService.getById(groupId);
                if (serbiaGroups == null) {
                    ResponseResult.output(HttpServletResponse.SC_NOT_FOUND);
                    return;
                }
                String fieldJsonData = (String) ReflectUtil.getFieldValue(serbiaGroups, fieldName);
                if (fieldJsonData == null && !cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_BAD_REQUEST);
                    return;
                }
                if (!BaseUpDownloader.containFile(fieldJsonData, filename)
                        && !cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_FORBIDDEN);
                    return;
                }
            }
            UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(SerbiaGroups.class, fieldName);
            if (!storeInfo.isSupportUpload()) {
                ResponseResult.output(HttpServletResponse.SC_NOT_IMPLEMENTED,
                        ResponseResult.error(ErrorCodeEnum.INVALID_UPLOAD_FIELD));
                return;
            }
            BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
            upDownloader.doDownload(appConfig.getUploadFileBaseDir(),
                    SerbiaGroups.class.getSimpleName(), fieldName, filename, asImage, response);
        } catch (Exception e) {
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 文件上传操作。
     *
     * @param fieldName  上传文件名。
     * @param asImage    是否作为图片上传。如果是图片，今后下载的时候无需权限验证。否则就是附件上传，下载时需要权限验证。
     * @param uploadFile 上传文件对象。
     */
    @SaCheckPermission("serbiaGroups.view")
    @OperationLog(type = SysOperationLogType.UPLOAD, saveResponse = false)
    @PostMapping("/upload")
    public void upload(
            @RequestParam String fieldName,
            @RequestParam Boolean asImage,
            @RequestParam("uploadFile") MultipartFile uploadFile) throws IOException {
        UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(SerbiaGroups.class, fieldName);
        // 这里就会判断参数中指定的字段，是否支持上传操作。
        if (!storeInfo.isSupportUpload()) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN,
                    ResponseResult.error(ErrorCodeEnum.INVALID_UPLOAD_FIELD));
            return;
        }
        // 根据字段注解中的存储类型，通过工厂方法获取匹配的上传下载实现类，从而解耦。
        BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
        UploadResponseInfo responseInfo = upDownloader.doUpload(null,
                appConfig.getUploadFileBaseDir(), SerbiaGroups.class.getSimpleName(), fieldName, asImage, uploadFile);
        if (Boolean.TRUE.equals(responseInfo.getUploadFailed())) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN,
                    ResponseResult.error(ErrorCodeEnum.UPLOAD_FAILED, responseInfo.getErrorMessage()));
            return;
        }
        cacheHelper.putSessionUploadFile(responseInfo.getFilename());
        ResponseResult.output(ResponseResult.success(responseInfo));
    }

    private ResponseResult<Void> doDelete(Long groupId) {
        String errorMessage;
        // 验证关联Id的数据合法性
        SerbiaGroups originalSerbiaGroups = serbiaGroupsService.getById(groupId);
        if (originalSerbiaGroups == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!serbiaGroupsService.remove(groupId)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
