package com.serbia.webadmin.app.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.serbia.common.log.annotation.OperationLog;
import com.serbia.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.serbia.webadmin.app.vo.*;
import com.serbia.webadmin.app.dto.*;
import com.serbia.webadmin.app.model.*;
import com.serbia.webadmin.app.service.*;
import com.serbia.common.core.object.*;
import com.serbia.common.core.util.*;
import com.serbia.common.core.constant.*;
import com.serbia.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 聊天信息管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-12-10
 */
@Tag(name = "聊天信息管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/serbiaMessages")
public class SerbiaMessagesController {

    @Autowired
    private SerbiaMessagesService serbiaMessagesService;

    /**
     * 新增聊天信息管理数据。
     *
     * @param serbiaMessagesDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {
            "serbiaMessagesDto.messageId",
            "serbiaMessagesDto.searchString",
            "serbiaMessagesDto.createTimeStart",
            "serbiaMessagesDto.createTimeEnd"})
    @SaCheckPermission("serbiaMessages.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody SerbiaMessagesDto serbiaMessagesDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(serbiaMessagesDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        SerbiaMessages serbiaMessages = MyModelUtil.copyTo(serbiaMessagesDto, SerbiaMessages.class);
        // 验证关联Id的数据合法性
        CallResult callResult = serbiaMessagesService.verifyRelatedData(serbiaMessages, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        serbiaMessages = serbiaMessagesService.saveNew(serbiaMessages);
        return ResponseResult.success(serbiaMessages.getMessageId());
    }

    /**
     * 更新聊天信息管理数据。
     *
     * @param serbiaMessagesDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {
            "serbiaMessagesDto.searchString",
            "serbiaMessagesDto.createTimeStart",
            "serbiaMessagesDto.createTimeEnd"})
    @SaCheckPermission("serbiaMessages.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody SerbiaMessagesDto serbiaMessagesDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(serbiaMessagesDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        SerbiaMessages serbiaMessages = MyModelUtil.copyTo(serbiaMessagesDto, SerbiaMessages.class);
        SerbiaMessages originalSerbiaMessages = serbiaMessagesService.getById(serbiaMessages.getMessageId());
        if (originalSerbiaMessages == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = serbiaMessagesService.verifyRelatedData(serbiaMessages, originalSerbiaMessages);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!serbiaMessagesService.update(serbiaMessages, originalSerbiaMessages)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除聊天信息管理数据。
     *
     * @param messageId 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("serbiaMessages.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long messageId) {
        if (MyCommonUtil.existBlankArgument(messageId)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(messageId);
    }

    /**
     * 批量删除聊天信息管理数据。
     *
     * @param messageIdList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("serbiaMessages.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> messageIdList) {
        if (MyCommonUtil.existBlankArgument(messageIdList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long messageId : messageIdList) {
            ResponseResult<Void> responseResult = this.doDelete(messageId);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的聊天信息管理列表。
     *
     * @param serbiaMessagesDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("serbiaMessages.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<SerbiaMessagesVo>> list(
            @MyRequestBody SerbiaMessagesDto serbiaMessagesDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        SerbiaMessages serbiaMessagesFilter = MyModelUtil.copyTo(serbiaMessagesDtoFilter, SerbiaMessages.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, SerbiaMessages.class);
        List<SerbiaMessages> serbiaMessagesList =
                serbiaMessagesService.getSerbiaMessagesListWithRelation(serbiaMessagesFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(serbiaMessagesList, SerbiaMessagesVo.class));
    }

    /**
     * 查看指定聊天信息管理对象详情。
     *
     * @param messageId 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("serbiaMessages.view")
    @GetMapping("/view")
    public ResponseResult<SerbiaMessagesVo> view(@RequestParam Long messageId) {
        SerbiaMessages serbiaMessages = serbiaMessagesService.getByIdWithRelation(messageId, MyRelationParam.full());
        if (serbiaMessages == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        SerbiaMessagesVo serbiaMessagesVo = MyModelUtil.copyTo(serbiaMessages, SerbiaMessagesVo.class);
        return ResponseResult.success(serbiaMessagesVo);
    }

    private ResponseResult<Void> doDelete(Long messageId) {
        String errorMessage;
        // 验证关联Id的数据合法性
        SerbiaMessages originalSerbiaMessages = serbiaMessagesService.getById(messageId);
        if (originalSerbiaMessages == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!serbiaMessagesService.remove(messageId)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
