package com.serbia.webadmin.app.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.serbia.webadmin.app.service.*;
import com.serbia.webadmin.app.dao.*;
import com.serbia.webadmin.app.model.*;
import com.serbia.common.core.base.dao.BaseDaoMapper;
import com.serbia.common.core.constant.GlobalDeletedFlag;
import com.serbia.common.core.object.MyRelationParam;
import com.serbia.common.core.object.CallResult;
import com.serbia.common.core.base.service.BaseService;
import com.serbia.common.core.util.MyModelUtil;
import com.serbia.common.sequence.wrapper.IdGeneratorWrapper;
import com.serbia.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 塞尔用户管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-12-10
 */
@Slf4j
@Service("serbiaUsersService")
public class SerbiaUsersServiceImpl extends BaseService<SerbiaUsers, Long> implements SerbiaUsersService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private SerbiaUsersMapper serbiaUsersMapper;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<SerbiaUsers> mapper() {
        return serbiaUsersMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public SerbiaUsers saveNew(SerbiaUsers serbiaUsers) {
        serbiaUsersMapper.insert(this.buildDefaultValue(serbiaUsers));
        return serbiaUsers;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<SerbiaUsers> serbiaUsersList) {
        if (CollUtil.isNotEmpty(serbiaUsersList)) {
            serbiaUsersList.forEach(this::buildDefaultValue);
            serbiaUsersMapper.insertList(serbiaUsersList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(SerbiaUsers serbiaUsers, SerbiaUsers originalSerbiaUsers) {
        MyModelUtil.fillCommonsForUpdate(serbiaUsers, originalSerbiaUsers);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<SerbiaUsers> uw = this.createUpdateQueryForNullValue(serbiaUsers, serbiaUsers.getUserId());
        return serbiaUsersMapper.update(serbiaUsers, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long userId) {
        return serbiaUsersMapper.deleteById(userId) == 1;
    }

    @Override
    public List<SerbiaUsers> getSerbiaUsersList(SerbiaUsers filter, String orderBy) {
        return serbiaUsersMapper.getSerbiaUsersList(filter, orderBy);
    }

    @Override
    public List<SerbiaUsers> getSerbiaUsersListWithRelation(SerbiaUsers filter, String orderBy) {
        List<SerbiaUsers> resultList = serbiaUsersMapper.getSerbiaUsersList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    /**
     * 修改用户头像。
     *
     * @param userId  用户主键Id。
     * @param newHeadImage 新的头像信息。
     * @return 成功返回true，否则false。
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean changeHeadImage(Long userId, String newHeadImage) {
        SerbiaUsers updatedUser = new SerbiaUsers();
        updatedUser.setUserId(userId);
        updatedUser.setHeadImageUrl(newHeadImage);
        return serbiaUsersMapper.updateById(updatedUser) == 1;
    }

    @Override
    public CallResult verifyRelatedData(SerbiaUsers serbiaUsers, SerbiaUsers originalSerbiaUsers) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(serbiaUsers, originalSerbiaUsers, SerbiaUsers::getSex)
                && !globalDictService.existDictItemFromCache("UserSex", serbiaUsers.getSex())) {
            return CallResult.error(String.format(errorMessageFormat, "性别(1:男 2:女 3:其它)"));
        }
        return CallResult.ok();
    }

    private SerbiaUsers buildDefaultValue(SerbiaUsers serbiaUsers) {
        if (serbiaUsers.getUserId() == null) {
            serbiaUsers.setUserId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(serbiaUsers);
        serbiaUsers.setDeletedFlag(GlobalDeletedFlag.NORMAL);
        return serbiaUsers;
    }
}
