package com.serbia.webadmin.websocket.handler;


import com.alibaba.fastjson.JSON;
import com.serbia.common.core.util.MyModelUtil;
import com.serbia.webadmin.app.dao.SerbiaMessagesMapper;
import com.serbia.webadmin.app.dto.SerbiaMessagesDto;
import com.serbia.webadmin.app.model.SerbiaMessages;
import com.serbia.webadmin.config.WebSocketAutoConfig;
import org.springframework.context.ApplicationContext;
import org.springframework.web.socket.*;
import org.springframework.web.socket.handler.BinaryWebSocketHandler;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class SingleWebSocketHandler  extends BinaryWebSocketHandler {
    private ApplicationContext applicationContext;

    private final static Map<String, WebSocketSession> sessionMap =  new ConcurrentHashMap<>();

    public  SingleWebSocketHandler(ApplicationContext applicationContext){
        this.applicationContext = applicationContext;
    }

    /**
     * websocket连接时调用的方法
     * @param session
     * @throws Exception
     */
    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {

        Map<String, Object> attributes = session.getAttributes();
        //从拦截器获取到放入map的id
        String id = attributes.get("id").toString();
        //判断连接集合里面是否存在该连接，不存在则放入
        if (!sessionMap.containsKey(id)){
            //放入连接保持长连接
            sessionMap.put(id,session);
        }
    }

    /**
     * 接收消息调用
     * @param session
     * @param message
     * @throws Exception
     */
    @Override
    public void handleMessage(WebSocketSession session, WebSocketMessage<?> message) throws Exception {

        //获取payload方法 也就是前端发来的消息
        SerbiaMessagesDto serbiaMessagesDto = JSON.parseObject(message.getPayload().toString(), SerbiaMessagesDto.class);
        //获取用户标识
        Long receiverId = serbiaMessagesDto.getReceiverId();
        //判断连接map中是否存在该连接 存在则发送消息
        if (sessionMap.containsKey(receiverId)){
            WebSocketSession webSocketSession = sessionMap.get(receiverId);
            sendMessage(webSocketSession,new TextMessage(serbiaMessagesDto.getMessageContent()));
        }
        //将消息入库 并标记为未读
        //applicationContext获取mapper mapper来入库
        SerbiaMessagesMapper bean = applicationContext.getBean(SerbiaMessagesMapper.class);
        bean.insert(MyModelUtil.copyTo(serbiaMessagesDto, SerbiaMessages.class));
        //如果用户在线 就直接发送 再入库
        //如果用户不在线 就放入mq等待消费（死循环）
        //会有消费者一直去消费 判断sessionMap用户是否在线 如果不在线 重新放入mq在线就直接消费，更改数据库状态 不再发送
        // 如果用户在线，直接发送消息后入库
        if (sessionMap.containsKey(receiverId)) {
            sendMessage(sessionMap.get(receiverId), new TextMessage(serbiaMessagesDto.getMessageContent()));
        } else {
//            // 如果用户不在线，就放入 MQ 等待消费（死循环）
//            MessageQueueService mqService = applicationContext.getBean(MessageQueueService.class);
//            mqService.sendMessageToQueue(messageEntity);
//
//            // 会有消费者一直去消费
//            while (true) {
//                SerbiaMessageEntity consumedMessage = mqService.consumeMessageFromQueue(receiverId);
//                if (sessionMap.containsKey(receiverId)) {
//                    // 用户在线，直接发送消息并更新数据库状态
//                    WebSocketSession webSocketSession = sessionMap.get(receiverId);
//                    sendMessage(webSocketSession, new TextMessage(consumedMessage.getMessageContent()));
//                    consumedMessage.setReadStatus(true); // 设置为已读
//                    serbiaMessagesMapper.update(consumedMessage);
//                    break;
//                } else {
//                    // 用户不在线，重新放入 MQ
//                    mqService.sendMessageToQueue(consumedMessage);
//                }
//            }
        }
    }

    /**
     * 异常调用
     * @param session
     * @param exception
     * @throws Exception
     */
    @Override
    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
        Map<String, Object> attributes = session.getAttributes();
        //一样获取到拦截器放入map的id
        String id = attributes.get("id").toString();
        //如果连接集合存在此id或者此链接 则删除
        if (sessionMap.containsKey(id) || session.isOpen()){
            sessionMap.remove(id);
        }
    }

    /**
     * 关闭调用
     * @param session
     * @param closeStatus
     * @throws Exception
     */
    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus closeStatus) throws Exception {
        Map<String, Object> attributes = session.getAttributes();
        //一样获取到拦截器放入map的id
        String id = attributes.get("id").toString();
        //如果连接集合存在此id或者此链接 则删除
        if (sessionMap.containsKey(id)||session.isOpen()){
            sessionMap.remove(id);
        }
    }

    /**
     * 是否支持部分消息
     * @return
     */
    @Override
    public boolean supportsPartialMessages() {
        return false;
    }

    /**
     * 发送消息
     * @param message
     * @throws Exception
     */
    public void sendMessage(WebSocketSession session,WebSocketMessage<?> message) throws Exception{
        session.sendMessage(message);
    }
}
