package com.serbia.webadmin.wechat.controller;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.api.WxMaUserService;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.dev33.satoken.annotation.SaIgnore;
import cn.dev33.satoken.session.SaSession;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.BooleanUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.serbia.common.core.annotation.DisableDataFilter;
import com.serbia.common.core.constant.ErrorCodeEnum;
import com.serbia.common.core.object.LoginUserInfo;
import com.serbia.common.core.object.ResponseResult;
import com.serbia.common.core.object.TokenData;
import com.serbia.common.core.upload.BaseUpDownloader;
import com.serbia.common.core.upload.UpDownloaderFactory;
import com.serbia.common.core.upload.UploadResponseInfo;
import com.serbia.common.core.upload.UploadStoreInfo;
import com.serbia.common.core.util.*;
import com.serbia.common.log.annotation.OperationLog;
import com.serbia.common.log.model.constant.SysOperationLogType;
import com.serbia.common.satoken.util.SaTokenUtil;
import com.serbia.webadmin.app.model.SerbiaUsers;
import com.serbia.webadmin.app.model.constant.UserStatus;
import com.serbia.webadmin.app.service.SerbiaUsersService;
import com.serbia.webadmin.app.vo.SerbiaUsersVo;
import com.serbia.webadmin.config.ApplicationConfig;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * 小程序登录控制器。
 *
 * @author huangwen
 * @date 2024/11/14
 */
@Tag(name = "小程序登录接口")
@Slf4j
@RestController
@DisableDataFilter
@SaIgnore
@RequestMapping("/wechat/login")
public class LoginToWechatController {

    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private SerbiaUsersService serUserService;
    @Autowired
    private WxMaService wxMaService;
    @Autowired
    private UpDownloaderFactory upDownloaderFactory;

    private static final String SHOW_NAME_FIELD = "showName";
    private static final String HEAD_IMAGE_URL_FIELD = "headImageUrl";


    /**
     * 小程序根据phoneCode登录
     */
    @SaIgnore
    @OperationLog(type = SysOperationLogType.LOGIN, saveResponse = false)
    @GetMapping("/doLoginByWxMini")
    public ResponseResult<JSONObject> doLoginByWxMini(String phoneCode) throws WxErrorException {
        log.info("小程序获取手机号code:" + phoneCode);
        WxMaUserService userService = wxMaService.getUserService();
        WxMaPhoneNumberInfo phoneNoInfo = userService.getPhoneNoInfo(phoneCode);

        log.info("小程序获取手机号信息:" + phoneNoInfo);
        SerbiaUsers serUser = serUserService.getOne(Wrappers.<SerbiaUsers>lambdaQuery().eq(SerbiaUsers::getMobile, phoneNoInfo.getPhoneNumber()));
        // 如果用户信息不存在，则新建
        if(serUser == null){
            serUser = new SerbiaUsers();
//            serUser.setUserId(new Snowflake(1,1).nextIdStr());
            serUser.setLoginName(phoneNoInfo.getPhoneNumber());
            serUser.setUserStatus(UserStatus.STATUS_NORMAL);
            serUser.setMobile(phoneNoInfo.getPhoneNumber());
            serUser.setPassword(passwordEncoder.encode("123456"));
            serUser.setCountryCode(phoneNoInfo.getCountryCode());
            serUser = serUserService.saveNew(serUser);
        }
        // 生成token
        JSONObject jsonObject = this.buildTourLoginDataAndLogin(serUser);
        jsonObject.put("userInfo", serUser);
        return ResponseResult.success(jsonObject);
    }

    private JSONObject buildTourLoginDataAndLogin(SerbiaUsers serUser) {
        TokenData tokenData = this.loginAndCreateToken(serUser);
        // 这里手动将TokenData存入request，便于OperationLogAspect统一处理操作日志。
        TokenData.addToRequest(tokenData);
        JSONObject jsonData = this.createResponseData(serUser);
        return jsonData;
    }

    private TokenData loginAndCreateToken(SerbiaUsers serUser) {
        String deviceType = MyCommonUtil.getDeviceTypeWithString();
        LoginUserInfo userInfo = BeanUtil.copyProperties(serUser, LoginUserInfo.class);
        String loginId = SaTokenUtil.makeLoginId(userInfo);
        StpUtil.login(loginId, deviceType);
        SaSession session = StpUtil.getTokenSession();
        TokenData tokenData = this.buildTokenData(serUser, session.getId(), StpUtil.getLoginDevice());
        String mySessionId = RedisKeyUtil.getSessionIdPrefix(tokenData, serUser.getLoginName()) + MyCommonUtil.generateUuid();
        tokenData.setMySessionId(mySessionId);
        tokenData.setToken(session.getToken());
        redissonClient.getBucket(mySessionId)
                .set(JSON.toJSONString(tokenData), appConfig.getSessionExpiredSeconds(), TimeUnit.SECONDS);
        session.set(TokenData.REQUEST_ATTRIBUTE_NAME, tokenData);
        return tokenData;
    }

    private JSONObject createResponseData(SerbiaUsers serUser) {
        JSONObject jsonData = new JSONObject();
        jsonData.put(TokenData.REQUEST_ATTRIBUTE_NAME, StpUtil.getTokenValue());
        jsonData.put(SHOW_NAME_FIELD, serUser.getShowName());
        if (StrUtil.isNotBlank(serUser.getHeadImageUrl())) {
            jsonData.put(HEAD_IMAGE_URL_FIELD, serUser.getHeadImageUrl());
        }
        return jsonData;
    }

    private TokenData buildTokenData(SerbiaUsers serUser, String sessionId, String deviceType) {
        TokenData tokenData = new TokenData();
        tokenData.setSessionId(sessionId);
        tokenData.setUserId(serUser.getUserId());
        tokenData.setLoginName(serUser.getLoginName());
        tokenData.setShowName(serUser.getShowName());
        tokenData.setLoginIp(IpUtil.getRemoteIpAddress(ContextUtil.getHttpRequest()));
        tokenData.setLoginTime(new Date());
        tokenData.setDeviceType(deviceType);
        tokenData.setHeadImageUrl(serUser.getHeadImageUrl());
        return tokenData;
    }


    /**
     * 上传并修改用户头像。
     *
     * @param uploadFile 上传的头像文件。
     */
    @PostMapping("/changeHeadImage")
    public void changeHeadImage(@RequestParam("uploadFile") MultipartFile uploadFile) throws IOException {
        UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(SerbiaUsers.class, HEAD_IMAGE_URL_FIELD);
        BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
        UploadResponseInfo responseInfo = upDownloader.doUpload(null,
                appConfig.getUploadFileBaseDir(), SerbiaUsers.class.getSimpleName(), HEAD_IMAGE_URL_FIELD, true, uploadFile);
        if (BooleanUtil.isTrue(responseInfo.getUploadFailed())) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN,
                    ResponseResult.error(ErrorCodeEnum.UPLOAD_FAILED, responseInfo.getErrorMessage()));
            return;
        }
        responseInfo.setDownloadUri("/admin/upms/login/downloadHeadImage");
        String newHeadImage = JSONArray.toJSONString(CollUtil.newArrayList(responseInfo));
        if (!serUserService.changeHeadImage(TokenData.takeFromRequest().getUserId(), newHeadImage)) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN, ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST));
            return;
        }
        ResponseResult.output(ResponseResult.success(responseInfo));
    }

    /**
     * 下载用户头像。
     *
     * @param filename 文件名。如果没有提供该参数，就从当前记录的指定字段中读取。
     * @param response Http 应答对象。
     */
    @GetMapping("/downloadHeadImage")
    public void downloadHeadImage(String filename, HttpServletResponse response) {
        try {
            UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(SerbiaUsers.class, HEAD_IMAGE_URL_FIELD);
            BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
            upDownloader.doDownload(appConfig.getUploadFileBaseDir(),
                    SerbiaUsers.class.getSimpleName(), HEAD_IMAGE_URL_FIELD, filename, true, response);
        } catch (Exception e) {
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 用户身份信息。
     *
     * @return 应答结果对象，包含查询结果。
     */
    @SaCheckPermission("user.view")
    @GetMapping("/userInfo")
    public ResponseResult<SerbiaUsersVo> userInfo() {
        TokenData tokenData = TokenData.takeFromRequest();
        if (tokenData == null || tokenData.getUserId() == null) {
            return ResponseResult.error(ErrorCodeEnum.UNAUTHORIZED_LOGIN);
        }
        log.info("token:" + tokenData);
        log.info("User ID: {}", tokenData.getUserId());
        SerbiaUsers user = new SerbiaUsers();
        user.setUserId(tokenData.getUserId());
        SerbiaUsers one = serUserService.getOne(user);
        if (one == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        SerbiaUsersVo userVo = MyModelUtil.copyTo(one, SerbiaUsersVo.class);

        return ResponseResult.success(userVo);
    }

}




